@@ -5,7 +5,7 @@
 # See emrun --help for more information
 
 from __future__ import print_function
-import os, platform, optparse, logging, re, pprint, atexit, subprocess, sys, time, struct, socket, cgi, tempfile, stat, shutil, json, uuid, shlex
+import os, platform, argparse, logging, re, pprint, atexit, subprocess, sys, time, struct, socket, cgi, tempfile, stat, shutil, json, uuid, shlex
 from operator import itemgetter
 from threading import Thread, RLock
 
@@ -63,18 +63,18 @@ ADB = ''
 # Host OS detection to autolocate browsers and other OS-specific support needs.
 WINDOWS = False
 LINUX = False
-OSX = False
+MACOS = False
 if os.name == 'nt':
   WINDOWS = True
 elif platform.system() == 'Linux':
   LINUX = True
 elif platform.mac_ver()[0] != '':
-  OSX = True
+  MACOS = True
 
   import plistlib
 
 # If you are running on an OS that is not any of these, must add explicit support for it.
-if not WINDOWS and not LINUX and not OSX:
+if not WINDOWS and not LINUX and not MACOS:
   raise Exception("Unknown OS!")
 
 import_win32api_modules_warned_once = False
@@ -99,7 +99,8 @@ def import_win32api_modules():
 
 # Returns wallclock time in seconds.
 def tick():
-  # Would like to return time.clock() since it's apparently better for precision, but it is broken on OSX 10.10 and Python 2.7.8.
+  # Would like to return time.clock() since it's apparently better for
+  # precision, but it is broken on macOS 10.10 and Python 2.7.8.
   return time.time()
 
 # Absolute wallclock time in seconds specifying when the previous HTTP stdout message from
@@ -285,6 +286,8 @@ user_pref("extensions.update.enabled", false);
 user_pref("extensions.getAddons.cache.enabled", false);
 // Enable wasm
 user_pref("javascript.options.wasm", true);
+// Enable SharedArrayBuffer (this profile is for a testing environment, so Spectre/Meltdown don't apply)
+user_pref("javascript.options.shared_memory", true);
 ''')
   f.close()
   logv('create_emrun_safe_firefox_profile: Created new Firefox profile "' + temp_firefox_profile_dir + '"')
@@ -310,7 +313,7 @@ def kill_browser_process():
     browser_process = None
     processname_killed_atexit = ''
     return
-  if len(processname_killed_atexit) > 0:
+  if len(processname_killed_atexit):
     if emrun_options.android:
       logv("Terminating Android app '" + processname_killed_atexit + "'.")
       subprocess.call([ADB, 'shell', 'am', 'force-stop', processname_killed_atexit])
@@ -371,26 +374,26 @@ class HTTPWebServer(socketserver.ThreadingMixIn, HTTPServer):
     with http_mutex:
       now = tick()
       max_message_queue_time = 5
-      if len(self.http_message_queue) > 0 and now - last_message_time > max_message_queue_time:
+      if len(self.http_message_queue) and now - last_message_time > max_message_queue_time:
         self.print_next_message()
   
   # Skips to printing the next message in queue now, independent of whether there was missed messages in the sequence numbering.
   def print_next_message(self):
     with http_mutex:
-      if len(self.http_message_queue) > 0:
+      if len(self.http_message_queue):
         self.expected_http_seq_num = self.http_message_queue[0][0]
         self.print_messages_due()
 
   # Completely flushes all out-of-order messages in the queue.
   def print_all_messages(self):
     with http_mutex:
-      while len(self.http_message_queue) > 0:
+      while len(self.http_message_queue):
         self.print_next_message()
 
   # Prints any messages that are now due after we logged some other previous messages.
   def print_messages_due(self):
     with http_mutex:
-      while len(self.http_message_queue) > 0:
+      while len(self.http_message_queue):
         msg = self.http_message_queue[0]
         if msg[0] == self.expected_http_seq_num:
           msg[2](msg[1])
@@ -452,7 +455,7 @@ class HTTPWebServer(socketserver.ThreadingMixIn, HTTPServer):
     self.print_all_messages()
 
   def handle_error(self, request, client_address):
-    err = sys.exc_info()[1][0]
+    err = sys.exc_info()[1].args[0]
     # Filter out the useless '[Errno 10054] An existing connection was forcibly closed by the remote host' errors that occur when we
     # forcibly kill the client.
     if err != 10054:
@@ -504,6 +507,8 @@ class HTTPHandler(SimpleHTTPRequestHandler):
       if guess_file_type.endswith('.'): guess_file_type = guess_file_type[:-1]
 
     ctype = self.guess_type(guess_file_type)
+    if guess_file_type.lower().endswith('.wasm'):
+      ctype = 'application/wasm'
     self.send_header("Content-type", ctype)
     fs = os.fstat(f.fileno())
     self.send_header("Content-Length", str(fs[6]))
@@ -560,6 +565,8 @@ class HTTPHandler(SimpleHTTPRequestHandler):
       return
     else:
       data = self.rfile.read(int(self.headers['Content-Length']))
+      if str is not bytes and isinstance(data, bytes):
+        data = data.decode('utf-8')
       data = data.replace("+", " ")
       data = unquote_u(data)
 
@@ -598,7 +605,15 @@ class HTTPHandler(SimpleHTTPRequestHandler):
     self.send_header('Connection','close')
     self.send_header('Expires','-1')
     self.end_headers()
-    self.wfile.write('OK')
+    self.wfile.write(b'OK')
+
+# Returns stdout by running command with universal_newlines=True
+def check_output(cmd, universal_newlines=True, *args, **kwargs):
+  if hasattr(subprocess, "run"):
+    return subprocess.run(cmd, universal_newlines=universal_newlines, stdout=subprocess.PIPE, check=True, *args, **kwargs).stdout
+  else:
+    # check_output is considered as an old API so prefer subprocess.run if possible
+    return subprocess.check_output(cmd, universal_newlines=universal_newlines, *args, **kwargs)
 
 # From http://stackoverflow.com/questions/4842448/getting-processor-information-in-python
 # Returns a string with something like "AMD64, Intel(R) Core(TM) i5-2557M CPU @ 1.70GHz, Intel64 Family 6 Model 42 Stepping 7, GenuineIntel"
@@ -612,20 +627,20 @@ def get_cpu_info():
       root_winmgmts = GetObject("winmgmts:root\cimv2")
       cpus = root_winmgmts.ExecQuery("Select * from Win32_Processor")
       cpu_name = cpus[0].Name + ', ' + platform.processor()
-      physical_cores = int(subprocess.check_output(['wmic', 'cpu', 'get', 'NumberOfCores']).split(b'\n')[1].strip())
-      logical_cores = int(subprocess.check_output(['wmic', 'cpu', 'get', 'NumberOfLogicalProcessors']).split(b'\n')[1].strip())
-      frequency = int(subprocess.check_output(['wmic', 'cpu', 'get', 'MaxClockSpeed']).split(b'\n')[1].strip())
-    elif OSX:
-      cpu_name = subprocess.check_output(['sysctl', '-n', 'machdep.cpu.brand_string']).strip()
-      physical_cores = int(subprocess.check_output(['sysctl', '-n', 'machdep.cpu.core_count']).strip())
-      logical_cores = int(subprocess.check_output(['sysctl', '-n', 'machdep.cpu.thread_count']).strip())
-      frequency = int(subprocess.check_output(['sysctl', '-n', 'hw.cpufrequency']).strip()) / 1000000
+      physical_cores = int(check_output(['wmic', 'cpu', 'get', 'NumberOfCores']).split('\n')[1].strip())
+      logical_cores = int(check_output(['wmic', 'cpu', 'get', 'NumberOfLogicalProcessors']).split('\n')[1].strip())
+      frequency = int(check_output(['wmic', 'cpu', 'get', 'MaxClockSpeed']).split('\n')[1].strip())
+    elif MACOS:
+      cpu_name = check_output(['sysctl', '-n', 'machdep.cpu.brand_string']).strip()
+      physical_cores = int(check_output(['sysctl', '-n', 'machdep.cpu.core_count']).strip())
+      logical_cores = int(check_output(['sysctl', '-n', 'machdep.cpu.thread_count']).strip())
+      frequency = int(check_output(['sysctl', '-n', 'hw.cpufrequency']).strip()) // 1000000
     elif LINUX:
-      all_info = subprocess.check_output(['cat', '/proc/cpuinfo']).strip()
+      all_info = check_output(['cat', '/proc/cpuinfo']).strip()
       for line in all_info.split("\n"):
         if "model name" in line:
           cpu_name = re.sub( ".*model name.*:", "", line, 1).strip()
-      lscpu = subprocess.check_output(['lscpu'])
+      lscpu = check_output(['lscpu'])
       frequency = int(float(re.search('CPU MHz: (.*)', lscpu).group(1).strip()) + 0.5)
       sockets = int(re.search('Socket\(s\): (.*)', lscpu).group(1).strip())
       physical_cores = sockets * int(re.search('Core\(s\) per socket: (.*)', lscpu).group(1).strip())
@@ -646,7 +661,7 @@ def get_cpu_info():
     }
 
 def get_android_cpu_infoline():
-  lines = subprocess.check_output([ADB, 'shell', 'cat', '/proc/cpuinfo']).split('\n')
+  lines = check_output([ADB, 'shell', 'cat', '/proc/cpuinfo']).split('\n')
   processor = ''
   hardware = ''
   for line in lines:
@@ -655,7 +670,7 @@ def get_android_cpu_infoline():
     elif line.startswith('Hardware'):
       hardware = line[line.find(':')+1:].strip()
 
-  freq = int(subprocess.check_output([ADB, 'shell', 'cat', '/sys/devices/system/cpu/cpu0/cpufreq/cpuinfo_max_freq']).strip())/1000
+  freq = int(check_output([ADB, 'shell', 'cat', '/sys/devices/system/cpu/cpu0/cpufreq/cpuinfo_max_freq']).strip())//1000
   return 'CPU: ' + processor + ', ' + hardware + ' @ ' + str(freq) + ' MHz'
 
 def win_get_gpu_info():
@@ -716,7 +731,7 @@ def win_get_gpu_info():
 def linux_get_gpu_info():
   glinfo = ''
   try:
-    glxinfo = subprocess.check_output('glxinfo')
+    glxinfo = check_output('glxinfo')
     for line in glxinfo.split("\n"):
       if "OpenGL vendor string:" in line: gl_vendor = line[len("OpenGL vendor string:"):].strip()
       if "OpenGL version string:" in line: gl_version = line[len("OpenGL version string:"):].strip()
@@ -727,7 +742,7 @@ def linux_get_gpu_info():
 
   adapterinfo = ''
   try:
-    vgainfo = subprocess.check_output(['lshw', '-C', 'display'], stderr=subprocess.PIPE)
+    vgainfo = check_output(['lshw', '-C', 'display'], stderr=subprocess.PIPE)
     vendor = re.search("vendor: (.*)", vgainfo).group(1).strip()
     product = re.search("product: (.*)", vgainfo).group(1).strip()
     description = re.search("description: (.*)", vgainfo).group(1).strip()
@@ -738,7 +753,7 @@ def linux_get_gpu_info():
 
   ram = 0
   try:
-    vgainfo = subprocess.check_output('lspci -v -s $(lspci | grep VGA | cut -d " " -f 1)', shell=True, stderr=subprocess.PIPE)
+    vgainfo = check_output('lspci -v -s $(lspci | grep VGA | cut -d " " -f 1)', shell=True, stderr=subprocess.PIPE)
     ram = int(re.search("\[size=([0-9]*)M\]", vgainfo).group(1)) * 1024 * 1024
   except Exception as e:
     logv(e)
@@ -747,10 +762,10 @@ def linux_get_gpu_info():
   if not model: model = 'Unknown'
   return [{'model': model, 'ram': ram}]
 
-def osx_get_gpu_info():
+def macos_get_gpu_info():
   gpus = []
   try:
-    info = subprocess.check_output(['system_profiler', 'SPDisplaysDataType'])
+    info = check_output(['system_profiler', 'SPDisplaysDataType'])
     info = info.split("Chipset Model:")[1:]
     for gpu in info:
       model_name = gpu.split('\n')[0].strip()
@@ -764,7 +779,7 @@ def osx_get_gpu_info():
 def get_gpu_info():
   if WINDOWS: return win_get_gpu_info()
   elif LINUX: return linux_get_gpu_info()
-  elif OSX: return osx_get_gpu_info()
+  elif MACOS: return macos_get_gpu_info()
   else: return []
 
 def get_executable_version(filename):
@@ -775,7 +790,7 @@ def get_executable_version(filename):
       ls = info['FileVersionLS']
       version = win32api.HIWORD(ms), win32api.LOWORD(ms), win32api.HIWORD(ls), win32api.LOWORD(ls)
       return '.'.join(map(str, version))
-    elif OSX:
+    elif MACOS:
       plistfile = filename[0:filename.find('MacOS')] + 'Info.plist'
       info = plistlib.readPlist(plistfile)
       # Data in Info.plists is a bit odd, this check combo gives best information on each browser.
@@ -787,7 +802,7 @@ def get_executable_version(filename):
         return info['CFBundleShortVersionString']
     elif LINUX:
       if 'firefox' in filename.lower():
-        version = subprocess.check_output([filename, '-v'])
+        version = check_output([filename, '-v'])
         version = version.replace('Mozilla Firefox ', '')
         return version.strip()
       else:
@@ -798,7 +813,7 @@ def get_executable_version(filename):
 
 def get_browser_build_date(filename):
   try:
-    if OSX:
+    if MACOS:
       plistfile = filename[0:filename.find('MacOS')] + 'Info.plist'
       info = plistlib.readPlist(plistfile)
       # Data in Info.plists is a bit odd, this check combo gives best information on each browser.
@@ -865,7 +880,7 @@ def win_get_file_properties(fname):
 
 def get_computer_model():
   try:
-    if OSX:
+    if MACOS:
       try:
         with open(os.path.join(os.getenv("HOME"), '.emrun.hwmodel.cached'), 'r') as f:
           model = f.read()
@@ -875,34 +890,34 @@ def get_computer_model():
 
       try:
         # http://apple.stackexchange.com/questions/98080/can-a-macs-model-year-be-determined-via-terminal-command
-        serial = subprocess.check_output(['system_profiler', 'SPHardwareDataType'])
+        serial = check_output(['system_profiler', 'SPHardwareDataType'])
         serial = re.search("Serial Number (.*): (.*)", serial)
         serial = serial.group(2).strip()[-4:]
         cmd = ['curl', '-s', 'http://support-sp.apple.com/sp/product?cc=' + serial]
         logv(str(cmd))
-        model = subprocess.check_output(cmd)
+        model = check_output(cmd)
         model = re.search('<configCode>(.*)</configCode>', model)
         model = model.group(1).strip()
         open(os.path.join(os.getenv("HOME"), '.emrun.hwmodel.cached'), 'w').write(model) # Cache the hardware model to disk
         return model
       except:
-        hwmodel = subprocess.check_output(['sysctl', 'hw.model'])
+        hwmodel = check_output(['sysctl', 'hw.model'])
         hwmodel = re.search('hw.model: (.*)', hwmodel).group(1).strip()
         return hwmodel
     elif WINDOWS:
-      manufacturer = subprocess.check_output(['wmic', 'baseboard', 'get', 'manufacturer']).split('\n')[1].strip()
-      version = subprocess.check_output(['wmic', 'baseboard', 'get', 'version']).split('\n')[1].strip()
-      product = subprocess.check_output(['wmic', 'baseboard', 'get', 'product']).split('\n')[1].strip()
+      manufacturer = check_output(['wmic', 'baseboard', 'get', 'manufacturer']).split('\n')[1].strip()
+      version = check_output(['wmic', 'baseboard', 'get', 'version']).split('\n')[1].strip()
+      product = check_output(['wmic', 'baseboard', 'get', 'product']).split('\n')[1].strip()
       if 'Apple' in manufacturer: return manufacturer + ' ' + version + ', ' + product
       else: return manufacturer + ' ' + product + ', ' + version
     elif LINUX:
-      board_vendor = subprocess.check_output(['cat', '/sys/devices/virtual/dmi/id/board_vendor']).strip()
-      board_name = subprocess.check_output(['cat', '/sys/devices/virtual/dmi/id/board_name']).strip()
-      board_version = subprocess.check_output(['cat', '/sys/devices/virtual/dmi/id/board_version']).strip()
+      board_vendor = check_output(['cat', '/sys/devices/virtual/dmi/id/board_vendor']).strip()
+      board_name = check_output(['cat', '/sys/devices/virtual/dmi/id/board_name']).strip()
+      board_version = check_output(['cat', '/sys/devices/virtual/dmi/id/board_version']).strip()
 
-      bios_vendor = subprocess.check_output(['cat', '/sys/devices/virtual/dmi/id/bios_vendor']).strip()
-      bios_version = subprocess.check_output(['cat', '/sys/devices/virtual/dmi/id/bios_version']).strip()
-      bios_date = subprocess.check_output(['cat', '/sys/devices/virtual/dmi/id/bios_date']).strip()
+      bios_vendor = check_output(['cat', '/sys/devices/virtual/dmi/id/bios_vendor']).strip()
+      bios_version = check_output(['cat', '/sys/devices/virtual/dmi/id/bios_version']).strip()
+      bios_date = check_output(['cat', '/sys/devices/virtual/dmi/id/bios_date']).strip()
       return board_vendor + ' ' + board_name + ' ' + board_version + ', ' + bios_vendor + ' ' + bios_version + ' (' + bios_date + ')'
   except Exception as e:
     logv(str(e))
@@ -918,14 +933,14 @@ def get_os_version():
 
       version = ''
       try:
-        version = ' ' + subprocess.check_output(['wmic', 'os', 'get', 'version']).split('\n')[1].strip()
+        version = ' ' + check_output(['wmic', 'os', 'get', 'version']).split('\n')[1].strip()
       except:
         pass
       return productName[0] + version + bitness
-    elif OSX:
-      return 'Mac OS ' + platform.mac_ver()[0] + bitness
+    elif MACOS:
+      return 'macOS ' + platform.mac_ver()[0] + bitness
     elif LINUX:
-      kernel_version = subprocess.check_output(['uname', '-r']).strip()
+      kernel_version = check_output(['uname', '-r']).strip()
       return ' '.join(platform.linux_distribution()) + ', linux kernel ' + kernel_version + ' ' + platform.architecture()[0] + bitness
   except:
     return 'Unknown OS'
@@ -936,7 +951,7 @@ def get_system_memory():
   try:
     if LINUX or emrun_options.android:
       if emrun_options.android:
-        lines = subprocess.check_output([ADB, 'shell', 'cat', '/proc/meminfo']).split('\n')
+        lines = check_output([ADB, 'shell', 'cat', '/proc/meminfo']).split('\n')
       else:
         mem = open('/proc/meminfo', 'r')
         lines = mem.readlines()
@@ -947,8 +962,8 @@ def get_system_memory():
           return int(sline[1]) * 1024
     elif WINDOWS:
       return win32api.GlobalMemoryStatusEx()['TotalPhys']
-    elif OSX:
-      return int(subprocess.check_output(['sysctl', '-n', 'hw.memsize']).strip())
+    elif MACOS:
+      return int(check_output(['sysctl', '-n', 'hw.memsize']).strip())
   except:
     return -1
 
@@ -987,7 +1002,7 @@ def win_get_default_browser():
       cmd = winreg.QueryValue(key, None)
       if cmd:
         parts = shlex.split(cmd)
-        if len(parts) > 0:
+        if len(parts):
           return [parts[0]]
   except WindowsError:
     logv("Unable to find default browser key in Windows registry. Trying fallback.")
@@ -999,7 +1014,7 @@ def win_get_default_browser():
 def find_browser(name):
   if WINDOWS and name == 'start':
     return win_get_default_browser()
-  if OSX and name == 'open':
+  if MACOS and name == 'open':
     return [name]
 
   if os.path.isfile(os.path.abspath(name)):
@@ -1016,7 +1031,7 @@ def find_browser(name):
     return [path_lookup]
 
   browser_locations = []
-  if OSX:
+  if MACOS:
     # Note: by default Firefox beta installs as 'Firefox.app', you must manually rename it to
     # FirefoxBeta.app after installation.
     browser_locations = [ ('firefox', '/Applications/Firefox.app/Contents/MacOS/firefox'),
@@ -1063,32 +1078,32 @@ def find_browser(name):
 
 def get_android_model():
   global ADB
-  manufacturer = subprocess.check_output([ADB, 'shell', 'getprop', 'ro.product.manufacturer']).strip()
-  brand = subprocess.check_output([ADB, 'shell', 'getprop', 'ro.product.brand']).strip()
-  model = subprocess.check_output([ADB, 'shell', 'getprop', 'ro.product.model']).strip()
-  board = subprocess.check_output([ADB, 'shell', 'getprop', 'ro.product.board']).strip()
-  device = subprocess.check_output([ADB, 'shell', 'getprop', 'ro.product.device']).strip()
-  name = subprocess.check_output([ADB, 'shell', 'getprop', 'ro.product.name']).strip()
+  manufacturer = check_output([ADB, 'shell', 'getprop', 'ro.product.manufacturer']).strip()
+  brand = check_output([ADB, 'shell', 'getprop', 'ro.product.brand']).strip()
+  model = check_output([ADB, 'shell', 'getprop', 'ro.product.model']).strip()
+  board = check_output([ADB, 'shell', 'getprop', 'ro.product.board']).strip()
+  device = check_output([ADB, 'shell', 'getprop', 'ro.product.device']).strip()
+  name = check_output([ADB, 'shell', 'getprop', 'ro.product.name']).strip()
   return manufacturer + ' ' + brand + ' ' + model + ' ' + board + ' ' + device + ' ' + name
 
 def get_android_os_version():
   global ADB
-  ver = subprocess.check_output([ADB, 'shell', 'getprop', 'ro.build.version.release']).strip()
-  apiLevel = subprocess.check_output([ADB, 'shell', 'getprop', 'ro.build.version.sdk']).strip()
+  ver = check_output([ADB, 'shell', 'getprop', 'ro.build.version.release']).strip()
+  apiLevel = check_output([ADB, 'shell', 'getprop', 'ro.build.version.sdk']).strip()
   if not apiLevel:
-    apiLevel = subprocess.check_output([ADB, 'shell', 'getprop', 'ro.build.version.sdk_int']).strip()
+    apiLevel = check_output([ADB, 'shell', 'getprop', 'ro.build.version.sdk_int']).strip()
 
   os = ''
   if ver:
     os += 'Android ' + ver + ' '
   if apiLevel:
     os += 'SDK API Level ' + apiLevel + ' '
-  os += subprocess.check_output([ADB, 'shell', 'getprop', 'ro.build.description']).strip()
+  os += check_output([ADB, 'shell', 'getprop', 'ro.build.description']).strip()
   return os
 
 def list_android_browsers():
   global ADB
-  apps = subprocess.check_output([ADB, 'shell', 'pm', 'list', 'packages', '-f']).replace('\r\n', '\n')
+  apps = check_output([ADB, 'shell', 'pm', 'list', 'packages', '-f']).replace('\r\n', '\n')
   browsers = []
   for line in apps.split('\n'):
     line = line.strip()
@@ -1188,7 +1203,7 @@ def get_system_info(format_json):
           }, indent=2)
     else:
       info = 'Model: ' + get_android_model() + '\n'
-      info += 'OS: ' + get_android_os_version() + ' with ' + str(get_system_memory()/1024/1024) + ' MB of System RAM\n'
+      info += 'OS: ' + get_android_os_version() + ' with ' + str(get_system_memory()//1024//1024) + ' MB of System RAM\n'
       info += 'CPU: ' + get_android_cpu_infoline() + '\n'
       return info.strip()
   else:
@@ -1217,94 +1232,106 @@ def get_system_info(format_json):
       gpus = get_gpu_info()
       info = 'Computer name: ' + socket.gethostname() + '\n' # http://stackoverflow.com/questions/799767/getting-name-of-windows-computer-running-python-script
       info += 'Model: ' + get_computer_model() + '\n'
-      info += 'OS: ' + get_os_version() + ' with ' + str(get_system_memory()/1024/1024) + ' MB of System RAM\n'
+      info += 'OS: ' + get_os_version() + ' with ' + str(get_system_memory()//1024//1024) + ' MB of System RAM\n'
       info += 'CPU: ' + cpu['model'] + ', ' + str(cpu['frequency']) + ' MHz, ' + str(cpu['physicalCores']) + ' physical cores, ' + str(cpu['logicalCores']) + ' logical cores\n';
       if len(gpus) == 1:
-        info += "GPU: " + gpus[0]['model'] + " with " + str(gpus[0]['ram']/1024/1024) + " MB of VRAM\n"
+        info += "GPU: " + gpus[0]['model'] + " with " + str(gpus[0]['ram']//1024//1024) + " MB of VRAM\n"
       elif len(gpus) > 1:
         for i in range(0, len(gpus)):
-          info += "GPU"+str(i)+ ": " + gpus[i]['model'] + " with " + str(gpus[i]['ram']/1024/1024) + " MBs of VRAM\n"
+          info += "GPU"+str(i)+ ": " + gpus[i]['model'] + " with " + str(gpus[i]['ram']//1024//1024) + " MBs of VRAM\n"
       info += 'UUID: ' + unique_system_id
       return info.strip()
 
+# Be resilient to quotes and whitespace
+def unwrap(s):
+  s = s.strip()
+  if (s.startswith('"') and s.endswith('"')) or (s.startswith("'") and s.endswith("'")):
+    s = s[1:-1].strip()
+  return s
+
 def run():
   global browser_process, browser_exe, processname_killed_atexit, emrun_options, emrun_not_enabled_nag_printed, ADB
-  usage_str = "usage: emrun [emrun_options] filename.html [html_cmdline_options]\n\n   where emrun_options specifies command line options for emrun itself, whereas\n   html_cmdline_options specifies startup arguments to the program."
-  parser = optparse.OptionParser(usage=usage_str)
+  usage_str = "emrun [emrun_options] filename.html [html_cmdline_options]\n\n   where emrun_options specifies command line options for emrun itself, whereas\n   html_cmdline_options specifies startup arguments to the program."
+  parser = argparse.ArgumentParser(usage=usage_str)
 
-  parser.add_option('--kill_start', dest='kill_on_start', action='store_true', default=False,
+  parser.add_argument('--kill_start', dest='kill_on_start', action='store_true', default=False,
     help='If true, any previously running instances of the target browser are killed before starting.')
 
-  parser.add_option('--kill_exit', dest='kill_on_exit', action='store_true', default=False,
+  parser.add_argument('--kill_exit', dest='kill_on_exit', action='store_true', default=False,
     help='If true, the spawned browser process is forcibly killed when it calls exit(). Note: Using this option may require explicitly passing the option --browser=/path/to/browser, to avoid emrun being detached from the browser process it spawns.')
 
-  parser.add_option('--no_server', dest='no_server', action='store_true', default=False,
+  parser.add_argument('--no_server', dest='no_server', action='store_true', default=False,
     help='If specified, a HTTP web server is not launched to host the page to run.')
 
-  parser.add_option('--no_browser', dest='no_browser', action='store_true', default=False,
+  parser.add_argument('--no_browser', dest='no_browser', action='store_true', default=False,
     help='If specified, emrun will not launch a web browser to run the page.')
 
-  parser.add_option('--no_emrun_detect', dest='no_emrun_detect', action='store_true', default=False,
+  parser.add_argument('--no_emrun_detect', dest='no_emrun_detect', action='store_true', default=False,
     help='If specified, skips printing the warning message if html page is detected to not have been built with --emrun linker flag.')
 
-  parser.add_option('--serve_after_close', dest='serve_after_close', action='store_true', default=False,
+  parser.add_argument('--serve_after_close', dest='serve_after_close', action='store_true', default=False,
     help='If true, serves the web page even after the application quits by user closing the web page.')
 
-  parser.add_option('--serve_after_exit', dest='serve_after_exit', action='store_true', default=False,
+  parser.add_argument('--serve_after_exit', dest='serve_after_exit', action='store_true', default=False,
     help='If true, serves the web page even after the application quits by a call to exit().')
 
-  parser.add_option('--serve_root', dest='serve_root', default='',
+  parser.add_argument('--serve_root', dest='serve_root', default='',
     help='If set, specifies the root path that the emrun web server serves. If not specified, the directory where the target .html page lives in is served.')
 
-  parser.add_option('--verbose', dest='verbose', action='store_true', default=False,
+  parser.add_argument('--verbose', dest='verbose', action='store_true', default=False,
     help='Enable verbose logging from emrun internal operation.')
 
-  parser.add_option('--hostname', dest='hostname', default=default_webserver_hostname,
+  parser.add_argument('--hostname', dest='hostname', default=default_webserver_hostname,
     help='Specifies the hostname the server runs in.')
 
-  parser.add_option('--port', dest='port', default=default_webserver_port, type="int",
+  parser.add_argument('--port', dest='port', default=default_webserver_port, type=int,
     help='Specifies the port the server runs in.')
 
-  parser.add_option('--log_stdout', dest='log_stdout', default='',
+  parser.add_argument('--log_stdout', dest='log_stdout', default='',
     help='Specifies a log filename where the browser process stdout data will be appended to.')
 
-  parser.add_option('--log_stderr', dest='log_stderr', default='',
+  parser.add_argument('--log_stderr', dest='log_stderr', default='',
     help='Specifies a log filename where the browser process stderr data will be appended to.')
 
-  parser.add_option('--silence_timeout', dest='silence_timeout', type="int", default=0,
+  parser.add_argument('--silence_timeout', dest='silence_timeout', type=int, default=0,
     help='If no activity is received in this many seconds, the browser process is assumed to be hung, and the web server is shut down and the target browser killed. Disabled by default.')
 
-  parser.add_option('--timeout', dest='timeout', type="int", default=0,
+  parser.add_argument('--timeout', dest='timeout', type=int, default=0,
     help='If the browser process does not quit or the page exit() in this many seconds, the browser is assumed to be hung, and the web server is shut down and the target browser killed. Disabled by default.')
 
-  parser.add_option('--timeout_returncode', dest='timeout_returncode', type="int", default=99999,
+  parser.add_argument('--timeout_returncode', dest='timeout_returncode', type=int, default=99999,
     help='Sets the exit code that emrun reports back to caller in the case that a page timeout occurs. Default: 99999.')
 
-  parser.add_option('--list_browsers', dest='list_browsers', action='store_true',
+  parser.add_argument('--list_browsers', dest='list_browsers', action='store_true',
     help='Prints out all detected browser that emrun is able to use with the --browser command and exits.')
 
-  parser.add_option('--browser', dest='browser', default='',
+  parser.add_argument('--browser', dest='browser', default='',
     help='Specifies the browser executable to run the web page in.')
 
-  parser.add_option('--android', dest='android', action='store_true', default=False,
+  parser.add_argument('--browser_args', dest='browser_args', default='',
+    help='Specifies the arguments to the browser executable.')
+
+  parser.add_argument('--android', dest='android', action='store_true', default=False,
     help='Launches the page in a browser of an Android device connected to an USB on the local system. (via adb)')
 
-  parser.add_option('--system_info', dest='system_info', action='store_true',
+  parser.add_argument('--system_info', dest='system_info', action='store_true',
     help='Prints information about the current system at startup.')
 
-  parser.add_option('--browser_info', dest='browser_info', action='store_true',
+  parser.add_argument('--browser_info', dest='browser_info', action='store_true',
     help='Prints information about the target browser to launch at startup.')
 
-  parser.add_option('--json', dest='json', action='store_true',
+  parser.add_argument('--json', dest='json', action='store_true',
     help='If specified, --system_info and --browser_info are outputted in JSON format.')
 
-  parser.add_option('--safe_firefox_profile', dest='safe_firefox_profile', action='store_true',
+  parser.add_argument('--safe_firefox_profile', dest='safe_firefox_profile', action='store_true',
     help='If true, the browser is launched into a new clean Firefox profile that is suitable for unattended automated runs. (If target browser != Firefox, this parameter is ignored)')
 
-  parser.add_option('--log_html', dest='log_html', action='store_true',
+  parser.add_argument('--log_html', dest='log_html', action='store_true',
     help='If set, information lines are printed out an HTML-friendly format.')
 
-  opts_with_param = ['--browser', '--timeout_returncode', '--timeout', '--silence_timeout', '--log_stderr', '--log_stdout', '--hostname', '--port', '--serve_root']
+  parser.add_argument('serve', nargs='*')
+
+  opts_with_param = ['--browser', '--browser_args', '--timeout_returncode', '--timeout', '--silence_timeout', '--log_stderr', '--log_stdout', '--hostname', '--port', '--serve_root']
 
   cmdlineparams = []
   # Split the startup arguments to two parts, delimited by the first (unbound) positional argument.
@@ -1319,7 +1346,8 @@ def run():
       break
     i += 1
 
-  (options, args) = parser.parse_args(sys.argv)
+  options = parser.parse_args(sys.argv[1:])
+  args = options.serve
   emrun_options = options
 
   if options.android:
@@ -1335,7 +1363,7 @@ def run():
       options.browser = which('xdg-open')
       if not options.browser:
         options.browser = 'firefox'
-    elif OSX:
+    elif MACOS:
       options.browser = 'safari'
 
   if options.list_browsers:
@@ -1345,16 +1373,16 @@ def run():
       list_pc_browsers()
     return
 
-  if len(args) < 2 and (options.system_info or options.browser_info):
+  if len(args) < 1 and (options.system_info or options.browser_info):
     options.no_server = options.no_browser = True # Don't run if only --system_info or --browser_info was passed.
 
-  if len(args) < 2 and not (options.no_server == True and options.no_browser == True):
+  if len(args) < 1 and not (options.no_server == True and options.no_browser == True):
     logi(usage_str)
     logi('')
     logi('Type emrun --help for a detailed list of available options.')
     return
 
-  file_to_serve = args[1] if len(args) > 1 else '.'
+  file_to_serve = args[0] if len(args) else '.'
   file_to_serve_is_url = file_to_serve.startswith('file://') or file_to_serve.startswith('http://') or file_to_serve.startswith('https://')
   
   if options.serve_root:
@@ -1366,7 +1394,7 @@ def run():
     url = file_to_serve
   else:
     url = os.path.relpath(os.path.abspath(file_to_serve), serve_dir)
-    if len(cmdlineparams) > 0:
+    if len(cmdlineparams):
       url += '?' + '&'.join(cmdlineparams)
     hostname = socket.gethostbyname(socket.gethostname()) if options.android else options.hostname
     url = 'http://' + hostname + ':' + str(options.port)+'/'+url
@@ -1421,10 +1449,7 @@ def run():
       processname_killed_atexit = browser_app[:browser_app.find('/')]
   else: #Launching a web page on local system.
     if options.browser:
-      # Be resilient to quotes and whitespace
-      options.browser = options.browser.strip()
-      if (options.browser.startswith('"') and options.browser.endswith('"')) or (options.browser.startswith("'") and options.browser.endswith("'")):
-        options.browser = options.browser[1:-1].strip()
+      options.browser = unwrap(options.browser)
 
     if not options.no_browser or options.browser_info:
       browser = find_browser(str(options.browser))
@@ -1432,26 +1457,26 @@ def run():
         loge('Unable to find browser "' + str(options.browser) + '"! Check the correctness of the passed --browser=xxx parameter!')
         return 1
       browser_exe = browser[0]
-      browser_args = []
+      browser_args = shlex.split(unwrap(options.browser_args))
 
       if 'safari' in browser_exe.lower():
         # Safari has a bug that a command line 'Safari http://page.com' does not launch that page,
         # but instead launches 'file:///http://page.com'. To remedy this, must use the open -a command
         # to run Safari, but unfortunately this will end up spawning Safari process detached from emrun.
-        if OSX:
+        if MACOS:
           browser = ['open', '-a', 'Safari'] + (browser[1:] if len(browser) > 1 else [])
 
         processname_killed_atexit = 'Safari'
       elif 'chrome' in browser_exe.lower():
         processname_killed_atexit = 'chrome'
-        browser_args = ['--incognito', '--enable-nacl', '--enable-pnacl', '--disable-restore-session-state', '--enable-webgl', '--no-default-browser-check', '--no-first-run', '--allow-file-access-from-files']
+        browser_args += ['--incognito', '--enable-nacl', '--enable-pnacl', '--disable-restore-session-state', '--enable-webgl', '--no-default-browser-check', '--no-first-run', '--allow-file-access-from-files']
     #    if options.no_server:
     #      browser_args += ['--disable-web-security']
       elif 'firefox' in browser_exe.lower():
         processname_killed_atexit = 'firefox'
       elif 'iexplore' in browser_exe.lower():
         processname_killed_atexit = 'iexplore'
-        browser_args = ['-private']
+        browser_args += ['-private']
       elif 'opera' in browser_exe.lower():
         processname_killed_atexit = 'opera'
 
@@ -1506,12 +1531,12 @@ def run():
 
   global browser_stdout_handle, browser_stderr_handle
   if options.log_stdout:
-    browser_stdout_handle = open(options.log_stdout, 'ab')
+    browser_stdout_handle = open(options.log_stdout, 'a')
   if options.log_stderr:
     if options.log_stderr == options.log_stdout:
       browser_stderr_handle = browser_stdout_handle
     else:
-      browser_stderr_handle = open(options.log_stderr, 'ab')
+      browser_stderr_handle = open(options.log_stderr, 'a')
 
   if not options.no_server:
     logv('Starting web server: http://%s:%i/' % (options.hostname, options.port))
