@@ -473,7 +473,7 @@ var LibraryGLFW = {
 
       if (event.target != Module["canvas"]) return;
 
-      eventButton = GLFW.DOMToGLFWMouseButton(event);
+      var eventButton = GLFW.DOMToGLFWMouseButton(event);
 
       if (status == 1) { // GLFW_PRESS
         GLFW.active.buttons |= (1 << eventButton);
@@ -610,7 +610,7 @@ var LibraryGLFW = {
     },
 
     requestFullScreen: function() {
-      Module.printErr('GLFW.requestFullScreen() is deprecated. Please call GLFW.requestFullscreen instead.');
+      err('GLFW.requestFullScreen() is deprecated. Please call GLFW.requestFullscreen instead.');
       GLFW.requestFullScreen = function() {
         return GLFW.requestFullscreen();
       }
@@ -627,7 +627,7 @@ var LibraryGLFW = {
     },
 
     cancelFullScreen: function() {
-      Module.printErr('GLFW.cancelFullScreen() is deprecated. Please call GLFW.exitFullscreen instead.');
+      err('GLFW.cancelFullScreen() is deprecated. Please call GLFW.exitFullscreen instead.');
       GLFW.cancelFullScreen = function() {
         return GLFW.exitFullscreen();
       }
@@ -1040,7 +1040,7 @@ var LibraryGLFW = {
     },
 
     GLFW2ParamToGLFW3Param: function(param) {
-      table = {
+      var table = {
         0x00030001:0, // GLFW_MOUSE_CURSOR
         0x00030002:0, // GLFW_STICKY_KEYS
         0x00030003:0, // GLFW_STICKY_MOUSE_BUTTONS
@@ -1551,7 +1551,7 @@ var LibraryGLFW = {
   glfwMakeContextCurrent: function(winid) {},
 
   glfwGetCurrentContext: function() {
-    return GLFW.active.id;
+    return GLFW.active ? GLFW.active.id : 0;
   },
 
   glfwSwapBuffers: function(winid) {
