@@ -24,7 +24,8 @@ namespace emscripten {
     namespace internal {
         typedef long GenericEnumValue;
 
-        typedef void (*GenericFunction)();
+        typedef void* GenericFunction;
+        typedef void (*VoidFunctionPtr)(void);
 
         // Implemented in JavaScript.  Don't call these directly.
         extern "C" {
@@ -221,7 +222,7 @@ namespace emscripten {
             void _embind_register_constant(
                 const char* name,
                 TYPEID constantType,
-                uintptr_t value);
+                double value);
         }
     }
 }
@@ -1012,12 +1013,12 @@ namespace emscripten {
             }
 
             template<typename ClassType>
-            static GenericFunction getUpcaster() {
+            static VoidFunctionPtr getUpcaster() {
                 return nullptr;
             }
 
             template<typename ClassType>
-            static GenericFunction getDowncaster() {
+            static VoidFunctionPtr getDowncaster() {
                 return nullptr;
             }
         };
@@ -1120,7 +1121,7 @@ namespace emscripten {
             BaseSpecifier::template verify<ClassType>();
 
             auto _getActualType = &getActualType<ClassType>;
-            auto upcast = BaseSpecifier::template getUpcaster<ClassType>();
+            auto upcast   = BaseSpecifier::template getUpcaster<ClassType>();
             auto downcast = BaseSpecifier::template getDowncaster<ClassType>();
             auto destructor = &raw_destructor<ClassType>;
 
@@ -1577,8 +1578,8 @@ namespace emscripten {
 
     namespace internal {
         template<typename T>
-        uintptr_t asGenericValue(T t) {
-            return static_cast<uintptr_t>(t);
+        double asGenericValue(T t) {
+            return static_cast<double>(t);
         }
 
         template<typename T>
@@ -1594,7 +1595,7 @@ namespace emscripten {
         _embind_register_constant(
             name,
             TypeID<const ConstantType&>::get(),
-            asGenericValue(BT::toWireType(v)));
+            static_cast<double>(asGenericValue(BT::toWireType(v))));
     }
 }
 
