@@ -1,5 +1,6 @@
 #!/usr/bin/env python2
 
+from __future__ import print_function
 import sys, shutil, os, json, tempfile, time
 
 profiler_logs_path = os.path.join(tempfile.gettempdir(), 'emscripten_toolchain_profiler_logs')
@@ -35,41 +36,41 @@ def create_profiling_graph():
   log_files = [f for f in list_files_in_directory(profiler_logs_path) if 'toolchain_profiler.pid_' in f]
 
   all_results = []
-  if len(log_files) > 0:
-    print 'Processing ' + str(len(log_files)) + ' profile log files in "' + profiler_logs_path + '"...'
+  if len(log_files):
+    print('Processing ' + str(len(log_files)) + ' profile log files in "' + profiler_logs_path + '"...')
   for f in log_files:
     try:
       json_data = open(f, 'r').read()
       lines = json_data.split('\n')
-      lines = filter(lambda x: x != '[' and x != ']' and x != ',' and len(x.strip()) > 0, lines)
-      lines = map(lambda x: (x + ',') if not x.endswith(',') else x, lines)
+      lines = [x for x in lines if x != '[' and x != ']' and x != ',' and len(x.strip())]
+      lines = [(x + ',') if not x.endswith(',') else x for x in lines]
       lines[-1] = lines[-1][:-1]
       json_data = '[' + '\n'.join(lines) + ']'
       all_results += json.loads(json_data)
-    except Exception, e:
-      print >> sys.stderr, str(e)
-      print >> sys.stderr, 'Failed to parse JSON file "' + f + '"!'
+    except Exception as e:
+      print(str(e), file=sys.stderr)
+      print('Failed to parse JSON file "' + f + '"!', file=sys.stderr)
       sys.exit(1)
   if len(all_results) == 0:
-    print 'No profiler logs were found in path "' + profiler_logs_path + '". Try setting the environment variable EM_PROFILE_TOOLCHAIN=1 and run some emcc commands, and then rerun "python emprofile.py --graph" again.'
+    print('No profiler logs were found in path "' + profiler_logs_path + '". Try setting the environment variable EM_PROFILE_TOOLCHAIN=1 and run some emcc commands, and then rerun "python emprofile.py --graph" again.')
     return
 
   all_results.sort(key=lambda x: x['time'])
 
   json_file = OUTFILE + '.json'
   open(json_file, 'w').write(json.dumps(all_results, indent=2))
-  print 'Wrote "' + json_file + '"'
+  print('Wrote "' + json_file + '"')
 
   html_file = OUTFILE + '.html'
   html_contents = open(os.path.join(os.path.dirname(os.path.realpath(__file__)), 'toolchain_profiler.results_template.html'), 'r').read().replace('{{{results_log_file}}}', '"' + json_file + '"')
   open(html_file, 'w').write(html_contents)
-  print 'Wrote "' + html_file + '"'
+  print('Wrote "' + html_file + '"')
 
   if not DEBUG_EMPROFILE_PY:
     delete_profiler_logs()
 
 if len(sys.argv) < 2:
-  print '''Usage:
+  print('''Usage:
        emprofile.py --reset
          Deletes all previously recorded profiling log files.
 
@@ -80,7 +81,7 @@ Optional parameters:
 
         --outfile=x.html
           Specifies the name of the results file to generate.
-'''
+''')
   sys.exit(1)
 
 if '--reset' in sys.argv:
@@ -88,5 +89,5 @@ if '--reset' in sys.argv:
 elif '--graph' in sys.argv:
   create_profiling_graph()
 else:
-  print 'Unknown command "' + sys.argv[1] + '"!'
+  print('Unknown command "' + sys.argv[1] + '"!')
   sys.exit(1)
