@@ -61,13 +61,15 @@ def get_native_optimizer():
     sys.exit(1)
 
   # Allow users to override the location of the optimizer executable by setting an environment variable EMSCRIPTEN_NATIVE_OPTIMIZER=/path/to/optimizer(.exe)
-  if os.environ.get('EMSCRIPTEN_NATIVE_OPTIMIZER') and len(os.environ.get('EMSCRIPTEN_NATIVE_OPTIMIZER')) > 0:
-    logging.debug('env forcing native optimizer at ' + os.environ.get('EMSCRIPTEN_NATIVE_OPTIMIZER'))
-    return os.environ.get('EMSCRIPTEN_NATIVE_OPTIMIZER')
+  opt = os.environ.get('EMSCRIPTEN_NATIVE_OPTIMIZER', '')
+  if len(opt):
+    logging.debug('env forcing native optimizer at ' + opt)
+    return opt
   # Also, allow specifying the location of the optimizer in .emscripten configuration file under EMSCRIPTEN_NATIVE_OPTIMIZER='/path/to/optimizer'
-  if hasattr(shared, 'EMSCRIPTEN_NATIVE_OPTIMIZER') and len(shared.EMSCRIPTEN_NATIVE_OPTIMIZER) > 0:
-    logging.debug('config forcing native optimizer at ' + shared.EMSCRIPTEN_NATIVE_OPTIMIZER)
-    return shared.EMSCRIPTEN_NATIVE_OPTIMIZER
+  opt = getattr(shared, 'EMSCRIPTEN_NATIVE_OPTIMIZER', '')
+  if len(opt):
+    logging.debug('config forcing native optimizer at ' + opt)
+    return opt
 
   FAIL_MARKER = shared.Cache.get_path('optimizer.building_failed')
   if os.path.exists(FAIL_MARKER):
@@ -241,7 +243,7 @@ class Minifier(object):
           (['--debug'] if source_map else []),
           stdout=subprocess.PIPE).stdout
 
-    assert len(output) > 0 and not output.startswith('Assertion failed'), 'Error in js optimizer: ' + output
+    assert len(output) and not output.startswith('Assertion failed'), 'Error in js optimizer: ' + output
     #print >> sys.stderr, "minified SHELL 3333333333333333", output, "\n44444444444444444444"
     code, metadata = output.split('// EXTRA_INFO:')
     self.globs = json.loads(metadata)
@@ -282,7 +284,7 @@ def run_on_chunk(command):
     proc = shared.run_process(command, stdout=subprocess.PIPE)
     output = proc.stdout
     assert proc.returncode == 0, 'Error in optimizer (return code ' + str(proc.returncode) + '): ' + output
-    assert len(output) > 0 and not output.startswith('Assertion failed'), 'Error in optimizer: ' + output
+    assert len(output) and not output.startswith('Assertion failed'), 'Error in optimizer: ' + output
     filename = temp_files.get(os.path.basename(filename) + '.jo.js').name
     f = open(filename, 'w')
     f.write(output)
@@ -345,7 +347,7 @@ def run_on_js(filename, passes, js_engine, source_map=False, extra_info=None, ju
         class Finals(object):
           buf = []
         def process(line):
-          if len(line) > 0 and (line.startswith(('Module[', 'if (globalScope)')) or line.endswith('["X"]=1;')):
+          if len(line) and (line.startswith(('Module[', 'if (globalScope)')) or line.endswith('["X"]=1;')):
             Finals.buf.append(line)
             return False
           return True
@@ -414,11 +416,11 @@ EMSCRIPTEN_FUNCS();
       # keep same chunks as before
       chunks = [f[1] for f in funcs]
 
-    chunks = [chunk for chunk in chunks if len(chunk) > 0]
-    if DEBUG and len(chunks) > 0: print('chunkification: num funcs:', len(funcs), 'actual num chunks:', len(chunks), 'chunk size range:', max(map(len, chunks)), '-', min(map(len, chunks)), file=sys.stderr)
+    chunks = [chunk for chunk in chunks if len(chunk)]
+    if DEBUG and len(chunks): print('chunkification: num funcs:', len(funcs), 'actual num chunks:', len(chunks), 'chunk size range:', max(map(len, chunks)), '-', min(map(len, chunks)), file=sys.stderr)
     funcs = None
 
-    if len(chunks) > 0:
+    if len(chunks):
       serialized_extra_info = suffix_marker + '\n'
       if minify_globals:
         serialized_extra_info += '// EXTRA_INFO:' + json.dumps(minify_info)
@@ -437,7 +439,7 @@ EMSCRIPTEN_FUNCS();
       filenames = []
 
   with ToolchainProfiler.profile_block('run_optimizer'):
-    if len(filenames) > 0:
+    if len(filenames):
       if not use_native(passes, source_map) or not get_native_optimizer():
         commands = [js_engine +
             [JS_OPTIMIZER, filename, 'noPrintMetadata'] +
@@ -505,9 +507,17 @@ EMSCRIPTEN_FUNCS();
       after = 'wakaUnknownAfter'
       start = coutput.find(after)
       end = coutput.find(')', start)
-      # First brace is from Closure Compiler comment, thus we need a second one
-      pre_2_second_brace = pre_2.find('{', pre_2.find('{')+1)
-      pre = coutput[:start] + '(/** @suppress {uselessCode} */ function(global,env,buffer) {\n' + pre_2[pre_2_second_brace+1:]
+      # If the closure comment to suppress useless code is present, we need to look one
+      # brace past it, as the first is in there. Otherwise, the first brace is the
+      # start of the function body (what we want).
+      USELESS_CODE_COMMENT = '/** @suppress {uselessCode} */ '
+      USELESS_CODE_COMMENT_BODY = 'uselessCode'
+      brace = pre_2.find('{') + 1
+      has_useless_code_comment = False
+      if pre_2[brace:brace + len(USELESS_CODE_COMMENT_BODY)] == USELESS_CODE_COMMENT_BODY:
+        brace = pre_2.find('{', brace) + 1
+        has_useless_code_comment = True
+      pre = coutput[:start] + '(' + (USELESS_CODE_COMMENT if has_useless_code_comment else '') + 'function(global,env,buffer) {\n' + pre_2[brace:]
       post = post_1 + end_asm + coutput[end+1:]
 
   with ToolchainProfiler.profile_block('write_pre'):
@@ -527,7 +537,7 @@ EMSCRIPTEN_FUNCS();
       if not os.environ.get('EMCC_NO_OPT_SORT'):
         funcs.sort(key=lambda x: (len(x[1]), x[0]), reverse=True)
 
-      if 'last' in passes and len(funcs) > 0:
+      if 'last' in passes and len(funcs):
         count = funcs[0][1].count('\n')
         if count > 3000:
           print('warning: Output contains some very large functions (%s lines in %s), consider building source files with -Os or -Oz, and/or trying OUTLINING_LIMIT to break them up (see settings.js; note that the parameter there affects AST nodes, while we measure lines here, so the two may not match up)' % (count, funcs[0][0]), file=sys.stderr)
